@@ -45,12 +45,12 @@ namespace Banshee.Query
         QueryFieldSet FieldSet { get; }
         string GetSqlSort (string key, bool asc);
     }
-    
+
     public class QueryDefines : IQueryDefines
     {
-        
+
     }*/
-    
+
     public static class BansheeQuery
     {
         private static bool asc = true;
@@ -63,12 +63,15 @@ namespace Banshee.Query
             CreateQueryOrder ("Album",      asc,  Catalog.GetString ("Album"), AlbumField),
             CreateQueryOrder ("Artist",     asc,  Catalog.GetString ("Artist"), ArtistField),
             // Translators: noun
-            CreateQueryOrder ("Title",      asc,  Catalog.GetString ("Title"), TitleField),
+            CreateQueryOrder ("Title",      asc,  Catalog.GetString ("Name"), TitleField),
             CreateQueryOrder ("Genre",      asc,  Catalog.GetString ("Genre"), GenreField),
             null,
             CreateQueryOrder ("Rating",     desc, Catalog.GetString ("Highest Rating"), RatingField),
             CreateQueryOrder ("Rating",     asc,  Catalog.GetString ("Lowest Rating"), RatingField),
             null,
+            CreateQueryOrder ("Score",      desc, Catalog.GetString ("Highest Score"), ScoreField),
+            CreateQueryOrder ("Score",      asc,  Catalog.GetString ("Lowest Score"), ScoreField),
+            null,
             CreateQueryOrder ("PlayCount",  desc, Catalog.GetString ("Most Often Played"), PlayCountField),
             CreateQueryOrder ("PlayCount",  asc,  Catalog.GetString ("Least Often Played"), PlayCountField),
             null,
@@ -86,21 +89,21 @@ namespace Banshee.Query
             new QueryLimit ("MB",      Catalog.GetString ("MB"), "CoreTracks.FileSize", (int) FileSizeFactor.MB),
             new QueryLimit ("GB",      Catalog.GetString ("GB"), "CoreTracks.FileSize", (int) FileSizeFactor.GB)
         };
-        
+
 #region QueryField Definitions
 
         public static QueryField ArtistField = new QueryField (
-            "artist", "ArtistName",
+            "artist", "DisplayArtistName",
             Catalog.GetString ("Artist"), "CoreArtists.NameLowered", true,
-            // Translators: These are unique search aliases for "artist".  Please, no spaces. Blank ok.
+            // Translators: These are unique search aliases for "artist". You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("artist"), Catalog.GetString ("by"), Catalog.GetString ("artists"),
             "by", "artist", "artists"
         );
 
         public static QueryField AlbumArtistField = new QueryField (
-            "albumartist", "AlbumArtist",
-            Catalog.GetString ("Album Artist"), "CoreAlbums.ArtistNameLowered", true,
-            // Translators: These are unique search aliases for "album".  Please, no spaces. Blank ok.
+            "albumartist", "DisplayAlbumArtistName",
+            Catalog.GetString ("Album Artist"), "CoreAlbums.ArtistNameLowered", false,
+            // Translators: These are unique search aliases for "album artist". You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("albumartist"), Catalog.GetString ("compilationartist"),
             "albumartist", "compilationartist"
         );
@@ -108,17 +111,17 @@ namespace Banshee.Query
         // TODO add IsCompilationField
 
         public static QueryField AlbumField = new QueryField (
-            "album", "AlbumTitle",
+            "album", "DisplayAlbumTitle",
             Catalog.GetString ("Album"), "CoreAlbums.TitleLowered", true,
-            // Translators: These are unique search aliases for "album".  Please, no spaces. Blank ok.
+            // Translators: These are unique search aliases for "album". You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("album"), Catalog.GetString ("on"), Catalog.GetString ("from"),
             "on", "album", "from", "albumtitle"
         );
 
-        public static QueryField DiscField = new QueryField (
+        public static QueryField DiscNumberField = new QueryField (
             "disc", "DiscNumber",
             Catalog.GetString ("Disc"), "CoreTracks.Disc", typeof(NaturalIntegerQueryValue),
-            // Translators: These are unique search fields (and nouns).  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields (and nouns). You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("disc"), Catalog.GetString ("cd"), Catalog.GetString ("discnum"),
             "disc", "cd", "discnum"
         );
@@ -126,16 +129,16 @@ namespace Banshee.Query
         public static QueryField DiscCountField = new QueryField (
             "disccount", "DiscCount",
             Catalog.GetString ("Disc Count"), "CoreTracks.DiscCount", typeof(NaturalIntegerQueryValue),
-            // Translators: These are unique search fields (and nouns).  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields (and nouns). You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("discs"), Catalog.GetString ("cds"),
             "discs", "cds"
         );
-        
+
         public static QueryField TrackNumberField = new QueryField (
             "track", "TrackNumber",
             // Translators: noun
-            Catalog.GetString ("Track"), "CoreTracks.TrackNumber", typeof(NaturalIntegerQueryValue),
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            Catalog.GetString ("Track Number"), "CoreTracks.TrackNumber", typeof(NaturalIntegerQueryValue),
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             "#", Catalog.GetString ("track"), Catalog.GetString ("trackno"), Catalog.GetString ("tracknum"),
             "track", "trackno", "tracknum"
         );
@@ -144,16 +147,15 @@ namespace Banshee.Query
             "trackcount", "TrackCount",
             // Translators: noun
             Catalog.GetString ("Track Count"), "CoreTracks.TrackCount", typeof(NaturalIntegerQueryValue),
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("tracks"), Catalog.GetString ("trackcount"),
             "tracks", "trackcount"
         );
 
         public static QueryField BpmField = new QueryField (
             "bpm", "Bpm",
-            // Translators: noun
-            Catalog.GetString ("BPM"), "CoreTracks.BPM", typeof(NaturalIntegerQueryValue),
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            Catalog.GetString ("Beats per Minute"), "CoreTracks.BPM", typeof(NaturalIntegerQueryValue),
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("bpm"),
             "bpm"
         );
@@ -162,15 +164,33 @@ namespace Banshee.Query
             "bitrate", "BitRate",
             // Translators: noun
             Catalog.GetString ("Bit Rate"), "CoreTracks.BitRate", typeof(NaturalIntegerQueryValue),
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("bitrate"), Catalog.GetString ("kbs"), Catalog.GetString ("kps"),
             "bitrate", "kbs", "kps"
         );
 
+        public static QueryField SampleRateField = new QueryField (
+            "samplerate", "SampleRate",
+            // Translators: noun
+            Catalog.GetString ("Sample Rate"), "CoreTracks.SampleRate", typeof(NaturalIntegerQueryValue),
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
+            Catalog.GetString ("samplerate"), Catalog.GetString ("Hz"),
+            "samplerate", "Hz"
+        );
+
+        public static QueryField BitsPerSampleField = new QueryField (
+            "bitspersample", "BitsPerSample",
+            // Translators: noun
+            Catalog.GetString ("Bits Per Sample"), "CoreTracks.BitsPerSample", typeof(NaturalIntegerQueryValue),
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
+            Catalog.GetString ("bitspersample"), Catalog.GetString ("bitdepth"), Catalog.GetString ("quantization"),
+            "bitspersample", "bitdepth", "quantization"
+        );
+
         public static QueryField TitleField = new QueryField (
-            "title", "TrackTitle",
-            Catalog.GetString ("Title"), "CoreTracks.TitleLowered", true,
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            "title", "DisplayTrackTitle",
+            Catalog.GetString ("Name"), "CoreTracks.TitleLowered", true,
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("title"), Catalog.GetString ("titled"), Catalog.GetString ("name"), Catalog.GetString ("named"),
             "title", "titled", "name", "named"
         );
@@ -178,7 +198,7 @@ namespace Banshee.Query
         public static QueryField YearField = new QueryField (
             "year", "Year",
             Catalog.GetString ("Year"), "CoreTracks.Year", typeof(YearQueryValue),
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("year"), Catalog.GetString ("released"), Catalog.GetString ("yr"),
             "year", "released", "yr"
         );
@@ -186,28 +206,28 @@ namespace Banshee.Query
         public static QueryField GenreField = new QueryField (
             "genre", "Genre",
             Catalog.GetString ("Genre"), "CoreTracks.Genre", false,
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("genre"), "genre"
         );
 
         public static QueryField ComposerField = new QueryField (
             "composer", "Composer",
             Catalog.GetString ("Composer"), "CoreTracks.Composer", false,
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("composer"), "composer"
         );
 
         public static QueryField ConductorField = new QueryField (
             "conductor", "Conductor",
             Catalog.GetString ("Conductor"), "CoreTracks.Conductor", false,
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("conductor"), "conductor"
         );
 
         public static QueryField GroupingField = new QueryField (
             "grouping", "Grouping",
             Catalog.GetString ("Grouping"), "CoreTracks.Grouping", false,
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("grouping"), "grouping"
         );
 
@@ -215,14 +235,23 @@ namespace Banshee.Query
             "comment", "Comment",
             // Translators: noun
             Catalog.GetString ("Comment"), "CoreTracks.Comment", false,
-            // Translators: These are unique search fields (and nouns).  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields (and nouns). You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("comment"), "comment"
         );
 
+        public static QueryField LicenseUriField = new QueryField (
+            "licenseuri", "LicenseUri",
+            // Translators: noun
+            Catalog.GetString ("License"), "CoreTracks.LicenseUri", typeof(ExactStringQueryValue),
+            // Translators: These are unique search fields (and nouns). You can use CSV for synonyms. Please, no spaces. Blank ok.
+            Catalog.GetString ("license"), Catalog.GetString ("licensed"), Catalog.GetString ("under"),
+            "license", "licensed", "under"
+        );
+
         public static QueryField RatingField = new QueryField (
             "rating", "SavedRating",
             Catalog.GetString ("Rating"), "CoreTracks.Rating", new Type [] {typeof(RatingQueryValue)},//, typeof(NullQueryValue)},
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("rating"), Catalog.GetString ("stars"),
             "rating", "stars"
         );
@@ -230,7 +259,7 @@ namespace Banshee.Query
         public static QueryField PlayCountField = new QueryField (
             "playcount", "PlayCount",
             Catalog.GetString ("Play Count"), "CoreTracks.PlayCount", typeof(NaturalIntegerQueryValue),
-            // Translators: These are unique search fields (and nouns).  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields (and nouns). You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("plays"), Catalog.GetString ("playcount"), Catalog.GetString ("listens"),
             "plays", "playcount", "numberofplays", "listens"
         );
@@ -238,7 +267,7 @@ namespace Banshee.Query
         public static QueryField SkipCountField = new QueryField (
             "skipcount", "SkipCount",
             Catalog.GetString ("Skip Count"), "CoreTracks.SkipCount", typeof(NaturalIntegerQueryValue),
-            // Translators: These are unique search fields (and nouns).  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields (and nouns). You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("skips"), Catalog.GetString ("skipcount"),
             "skips", "skipcount"
         );
@@ -246,15 +275,15 @@ namespace Banshee.Query
         public static QueryField FileSizeField = new QueryField (
             "filesize", "FileSize",
             Catalog.GetString ("File Size"), "CoreTracks.FileSize", typeof(FileSizeQueryValue),
-            // Translators: These are unique search fields (and nouns).  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields (and nouns). You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("size"), Catalog.GetString ("filesize"),
             "size", "filesize"
         );
 
         public static QueryField UriField = new QueryField (
             "uri", "Uri",
-            Catalog.GetString ("File Location"), "CoreTracks.Uri",
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            Catalog.GetString ("File Location"), "CoreTracks.Uri", typeof(ExactStringQueryValue),
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("uri"), Catalog.GetString ("path"), Catalog.GetString ("file"), Catalog.GetString ("location"),
             "uri", "path", "file", "location"
         );
@@ -262,15 +291,15 @@ namespace Banshee.Query
         public static QueryField DurationField = new QueryField (
             "duration", "Duration",
             Catalog.GetString ("Time"), "CoreTracks.Duration", typeof(TimeSpanQueryValue),
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("duration"), Catalog.GetString ("length"), Catalog.GetString ("time"),
             "duration", "length", "time"
         );
 
         public static QueryField MimeTypeField = new QueryField (
             "mimetype", "MimeType",
-            Catalog.GetString ("Mime Type"), "CoreTracks.MimeType {0} OR CoreTracks.Uri {0}",
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            Catalog.GetString ("Mime Type"), "CoreTracks.MimeType {0} OR CoreTracks.Uri {0}", typeof(ExactStringQueryValue),
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("type"), Catalog.GetString ("mimetype"), Catalog.GetString ("format"), Catalog.GetString ("ext"),
             "type", "mimetype", "format", "ext", "mime"
         );
@@ -278,7 +307,7 @@ namespace Banshee.Query
         public static QueryField LastPlayedField = new QueryField (
             "lastplayed", "LastPlayed",
             Catalog.GetString ("Last Played"), "CoreTracks.LastPlayedStamp", new Type [] {typeof(RelativeTimeSpanQueryValue), typeof(DateQueryValue)},
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("lastplayed"), Catalog.GetString ("played"), Catalog.GetString ("playedon"),
             "lastplayed", "played", "playedon"
         );
@@ -286,7 +315,7 @@ namespace Banshee.Query
         public static QueryField LastSkippedField = new QueryField (
             "lastskipped", "LastSkipped",
             Catalog.GetString ("Last Skipped"), "CoreTracks.LastSkippedStamp", new Type [] {typeof(RelativeTimeSpanQueryValue), typeof(DateQueryValue)},
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("lastskipped"), Catalog.GetString ("skipped"), Catalog.GetString ("skippedon"),
             "lastskipped", "skipped", "skippedon"
         );
@@ -294,7 +323,7 @@ namespace Banshee.Query
         public static QueryField DateAddedField = new QueryField (
             "added", "DateAdded",
             Catalog.GetString ("Date Added"), "CoreTracks.DateAddedStamp", new Type [] {typeof(RelativeTimeSpanQueryValue), typeof(DateQueryValue)},
-            // Translators: These are unique search fields.  Please, no spaces. Blank ok.
+            // Translators: These are unique search fields. You can use CSV for synonyms. Please, no spaces. Blank ok.
             Catalog.GetString ("added"), Catalog.GetString ("imported"), Catalog.GetString ("addedon"), Catalog.GetString ("dateadded"), Catalog.GetString ("importedon"),
             "added", "imported", "addedon", "dateadded", "importedon"
         );
@@ -310,17 +339,48 @@ namespace Banshee.Query
             "CoreTracks.TrackID {2} IN (SELECT TrackID FROM CoreSmartPlaylistEntries WHERE SmartPlaylistID = {1})", typeof(SmartPlaylistQueryValue),
             "smartplaylistid", "smartplaylist"
         );
-        
+
+        public static QueryField ScoreField = new QueryField (
+            "score", "Score",
+            Catalog.GetString ("Score"), "CoreTracks.Score", typeof(IntegerQueryValue),
+            //Translators: These are unique search fields (and nouns). You can use CSV for synonyms. Please, no spaces. Blank ok.
+            Catalog.GetString ("score"),
+            "score"
+        );
+
+        public static QueryField PlaybackErrorField = new QueryField (
+            "playbackerror", "PlaybackError",
+            Catalog.GetString ("Playback Error"), "CoreTracks.LastStreamError", typeof(PlaybackErrorQueryValue),
+            //Translators: These are unique search fields (and nouns). Please, no spaces. Blank ok.
+            Catalog.GetString ("playbackerror"),
+            "playbackerror", "error"
+        );
+
 #endregion
 
         public static QueryFieldSet FieldSet = new QueryFieldSet (
-            ArtistField, AlbumField, AlbumArtistField, TitleField, TrackNumberField, TrackCountField, DiscField, DiscCountField,
-            YearField, GenreField, ComposerField, ConductorField, GroupingField, CommentField, RatingField, PlayCountField,
+            ArtistField, AlbumField, AlbumArtistField, TitleField, TrackNumberField, TrackCountField, DiscNumberField, DiscCountField,
+            YearField, GenreField, ComposerField, ConductorField, GroupingField, CommentField, LicenseUriField, RatingField, PlayCountField,
             SkipCountField, FileSizeField, UriField, DurationField, MimeTypeField, LastPlayedField, LastSkippedField,
-            BpmField, BitRateField, DateAddedField, PlaylistField, SmartPlaylistField
+            BpmField, BitRateField, SampleRateField, BitsPerSampleField, DateAddedField, PlaylistField, SmartPlaylistField, ScoreField, PlaybackErrorField
         );
 
-        private const string default_sort = @"CoreAlbums.ArtistNameLowered ASC, CoreAlbums.TitleLowered ASC, CoreTracks.Disc ASC, CoreTracks.TrackNumber ASC";
+        // Type Initializer
+        static BansheeQuery ()
+        {
+            // Translators: noun
+            BpmField.ShortLabel         = Catalog.GetString ("BPM");
+            SkipCountField.ShortLabel   = Catalog.GetString ("Skips");
+            PlayCountField.ShortLabel   = Catalog.GetString ("Plays");
+
+            default_sort = String.Format (default_sort_template, "");
+            default_sort_by_year = String.Format (default_sort_template, "CoreTracks.Year ASC, ");
+        }
+
+        private const string default_sort_template = @"CoreAlbums.ArtistNameSortKey ASC, {0}CoreAlbums.TitleSortKey ASC, CoreTracks.Disc ASC, CoreTracks.TrackNumber ASC";
+        private static readonly string default_sort;
+        private static readonly string default_sort_by_year;
+
         public static string GetSort (string key)
         {
             return GetSort (key, false);
@@ -328,72 +388,105 @@ namespace Banshee.Query
 
         public static string GetSort (string key, bool asc)
         {
+            bool sort_by_year = Banshee.Configuration.Schema.LibrarySchema.SortByAlbumYear.Get ();
+            string sort_by_year_part = sort_by_year ? "CoreTracks.Year ASC," : "";
+            string sort = sort_by_year ? default_sort_by_year : default_sort;
+
             string ascDesc = asc ? "ASC" : "DESC";
             string sort_query = null;
             // TODO use the QueryFields here instead of matching on a string key
             string column = null;
             switch (key.ToLower ()) {
                 case "track":
+                case "grouping":
                     sort_query = String.Format (@"
-                        CoreAlbums.ArtistNameLowered ASC, 
-                        CoreAlbums.TitleLowered ASC, 
+                        CoreAlbums.ArtistNameSortKey ASC,
+                        {1}
+                        CoreAlbums.TitleSortKey ASC,
                         CoreTracks.Disc ASC,
-                        CoreTracks.TrackNumber {0}", ascDesc); 
+                        CoreTracks.TrackNumber {0}", ascDesc, sort_by_year_part);
                     break;
 
                 case "albumartist":
                     sort_query = String.Format (@"
-                        CoreAlbums.ArtistNameLowered {0}, 
-                        CoreAlbums.TitleLowered ASC, 
+                        CoreAlbums.ArtistNameSortKey {0},
+                        {1}
+                        CoreAlbums.TitleSortKey ASC,
                         CoreTracks.Disc ASC,
-                        CoreTracks.TrackNumber ASC", ascDesc); 
+                        CoreTracks.TrackNumber ASC", ascDesc, sort_by_year_part);
                     break;
 
                 case "artist":
                     sort_query = String.Format (@"
-                        CoreArtists.NameLowered {0}, 
-                        CoreAlbums.TitleLowered ASC,
+                        CoreArtists.NameSortKey {0},
+                        {1}
+                        CoreAlbums.TitleSortKey ASC,
                         CoreTracks.Disc ASC,
-                        CoreTracks.TrackNumber ASC", ascDesc); 
+                        CoreTracks.TrackNumber ASC", ascDesc, sort_by_year_part);
                     break;
 
                 case "album":
                     sort_query = String.Format (@"
-                        CoreAlbums.TitleLowered {0},
+                        CoreAlbums.TitleSortKey {0},
+                        {1}
                         CoreTracks.Disc ASC,
-                        CoreTracks.TrackNumber ASC", ascDesc); 
+                        CoreTracks.TrackNumber ASC", ascDesc, sort_by_year_part);
                     break;
 
                 case "title":
                     sort_query = String.Format (@"
-                        CoreTracks.TitleLowered {0},
-                        CoreAlbums.ArtistNameLowered ASC, 
-                        CoreAlbums.TitleLowered ASC", ascDesc); 
+                        CoreTracks.TitleSortKey {0},
+                        CoreAlbums.ArtistNameSortKey ASC,
+                        {1}
+                        CoreAlbums.TitleSortKey ASC", ascDesc, sort_by_year_part);
                     break;
 
                 case "random":
                     sort_query = "RANDOM ()";
                     break;
 
+                case "disc":
+                    sort_query = String.Format (@"
+                        CoreAlbums.ArtistNameSortKey ASC,
+                        {1}
+                        CoreAlbums.TitleSortKey ASC,
+                        CoreTracks.Disc {0},
+                        CoreTracks.TrackNumber ASC", ascDesc, sort_by_year_part);
+                    break;
+
                 // FIXME hacks to aid in migration of these sort keys to actually
                 // using the QueryField (or at least their .Names)
                 case "lastplayed":
                 case "lastskipped":
                     column = String.Format ("{0}stamp", key);
-                    goto case "comment";
+                    goto case "year";
                 case "added":
                     column = "dateaddedstamp";
-                    goto case "comment";
+                    goto case "year";
+
+                case "conductor":
+                case "genre":
+                case "composer":
+                case "comment":
+                    sort_query = String.Format (
+                        "HYENA_COLLATION_KEY(CoreTracks.{0}) {1}, {2}",
+                        column ?? key, ascDesc, sort
+                    );
+                    break;
+
+                case "score":
+                    sort_query = String.Format (@"
+                        CoreTracks.Score {0},
+                        CoreTracks.Playcount {0}, {1}", ascDesc, sort);
+                    break;
 
                 case "year":
                 case "bitrate":
+                case "samplerate":
+                case "bitspersample":
                 case "bpm":
-                case "conductor":
-                case "grouping":
                 case "trackcount":
-                case "disc":
                 case "disccount":
-                case "genre":
                 case "duration":
                 case "rating":
                 case "playcount":
@@ -404,11 +497,10 @@ namespace Banshee.Query
                 case "dateaddedstamp":
                 case "uri":
                 case "mimetype":
-                case "composer":
-                case "comment":
+                case "licenseuri":
                     sort_query = String.Format (
                         "CoreTracks.{0} {1}, {2}",
-                        column ?? key, ascDesc, default_sort
+                        column ?? key, ascDesc, sort
                     );
                     break;
                 default:
