@@ -31,12 +31,14 @@ using System.Collections.Generic;
 using Mono.Unix;
 
 using Hyena;
+using Hyena.Jobs;
 using Hyena.Data;
 using Hyena.Query;
 using Hyena.Data.Sqlite;
 using Hyena.Collections;
 
 using Banshee.Base;
+using Banshee.Preferences;
 using Banshee.ServiceStack;
 using Banshee.Configuration;
 using Banshee.Sources;
@@ -72,24 +74,42 @@ namespace Banshee.Sources
     }
 
     public delegate bool TrackEqualHandler (DatabaseTrackInfo a, TrackInfo b);
+    public delegate object TrackExternalObjectHandler (DatabaseTrackInfo a);
+    public delegate string TrackArtworkIdHandler (DatabaseTrackInfo a);
 
     public abstract class PrimarySource : DatabaseSource, IDisposable
     {
+        #region Functions that let us override some behavior of our DatabaseTrackInfos
+
         private TrackEqualHandler track_equal_handler;
         public TrackEqualHandler TrackEqualHandler {
             get { return track_equal_handler; }
             protected set { track_equal_handler = value; }
         }
-    
+
+        private TrackExternalObjectHandler track_external_object_handler;
+        public TrackExternalObjectHandler TrackExternalObjectHandler {
+            get { return track_external_object_handler; }
+            protected set { track_external_object_handler = value; }
+        }
+
+        private TrackArtworkIdHandler track_artwork_id_handler;
+        public TrackArtworkIdHandler TrackArtworkIdHandler {
+            get { return track_artwork_id_handler; }
+            protected set { track_artwork_id_handler = value; }
+        }
+
+        #endregion
+
         protected ErrorSource error_source;
         protected bool error_source_visible = false;
 
         protected string remove_range_sql = @"
-            INSERT INTO CoreRemovedTracks SELECT ?, TrackID, Uri FROM CoreTracks WHERE TrackID IN (SELECT {0});
+            INSERT INTO CoreRemovedTracks (DateRemovedStamp, TrackID, Uri) SELECT ?, TrackID, Uri FROM CoreTracks WHERE TrackID IN (SELECT {0});
             DELETE FROM CoreTracks WHERE TrackID IN (SELECT {0})";
 
         protected HyenaSqliteCommand remove_list_command = new HyenaSqliteCommand (@"
-            INSERT INTO CoreRemovedTracks SELECT ?, TrackID, Uri FROM CoreTracks WHERE TrackID IN (SELECT ItemID FROM CoreCache WHERE ModelID = ?);
+            INSERT INTO CoreRemovedTracks (DateRemovedStamp, TrackID, Uri) SELECT ?, TrackID, Uri FROM CoreTracks WHERE TrackID IN (SELECT ItemID FROM CoreCache WHERE ModelID = ?);
             DELETE FROM CoreTracks WHERE TrackID IN (SELECT ItemID FROM CoreCache WHERE ModelID = ?)
         ");
 
@@ -97,7 +117,7 @@ namespace Banshee.Sources
             DELETE FROM CoreArtists WHERE ArtistID NOT IN (SELECT ArtistID FROM CoreTracks);
             DELETE FROM CoreAlbums WHERE AlbumID NOT IN (SELECT AlbumID FROM CoreTracks)
         ");
-        
+
         protected HyenaSqliteCommand purge_tracks_command = new HyenaSqliteCommand (@"
             DELETE FROM CoreTracks WHERE PrimarySourceId = ?
         ");
@@ -113,18 +133,18 @@ namespace Banshee.Sources
                 if (dbid > 0) {
                     return dbid;
                 }
-                
+
                 dbid = ServiceManager.DbConnection.Query<int> ("SELECT PrimarySourceID FROM CorePrimarySources WHERE StringID = ?", UniqueId);
                 if (dbid == 0) {
                     dbid = ServiceManager.DbConnection.Execute ("INSERT INTO CorePrimarySources (StringID) VALUES (?)", UniqueId);
                 } else {
                     SavedCount = ServiceManager.DbConnection.Query<int> ("SELECT CachedCount FROM CorePrimarySources WHERE PrimarySourceID = ?", dbid);
                 }
-                
+
                 if (dbid == 0) {
                     throw new ApplicationException ("dbid could not be resolved, this should never happen");
                 }
-                
+
                 return dbid;
             }
         }
@@ -157,6 +177,20 @@ namespace Banshee.Sources
             protected set { is_local = value; }
         }
 
+        private static SourceSortType[] sort_types = new SourceSortType[] {
+            SortNameAscending,
+            SortSizeAscending,
+            SortSizeDescending
+        };
+
+        public override SourceSortType[] ChildSortTypes {
+            get { return sort_types; }
+        }
+
+        public override SourceSortType DefaultChildSort {
+            get { return SortNameAscending; }
+        }
+
         public delegate void TrackEventHandler (Source sender, TrackEventArgs args);
 
         public event TrackEventHandler TracksAdded;
@@ -168,30 +202,20 @@ namespace Banshee.Sources
         {
             return (primary_sources.ContainsKey (id)) ? primary_sources[id] : null;
         }
-        
-        public virtual SafeUri UriAndTypeToSafeUri (TrackUriType type, string uri_field)
-        {
-            if (type == TrackUriType.RelativePath && BaseDirectory != null)
-                return new SafeUri (System.IO.Path.Combine (BaseDirectory, uri_field));
-            else
-                return new SafeUri (uri_field);
-        }
-
-        public virtual void UriToFields (SafeUri uri, out TrackUriType type, out string uri_field)
-        {
-            uri_field = Paths.MakePathRelative (uri.AbsolutePath, BaseDirectory);
-            type = (uri_field == null) ? TrackUriType.AbsoluteUri : TrackUriType.RelativePath;
-            if (uri_field == null) {
-                uri_field = uri.AbsoluteUri;
-            }
-        }
 
         public virtual string BaseDirectory {
             get { return null; }
+            protected set { base_dir_with_sep = null; }
+        }
+
+        private string base_dir_with_sep;
+        public string BaseDirectoryWithSeparator {
+            get { return base_dir_with_sep ?? (base_dir_with_sep = BaseDirectory + System.IO.Path.DirectorySeparatorChar); }
         }
 
         protected PrimarySource (string generic_name, string name, string id, int order) : base (generic_name, name, id, order)
         {
+            Properties.SetString ("SortChildrenActionLabel", Catalog.GetString ("Sort Playlists by"));
             PrimarySourceInitialize ();
         }
 
@@ -215,6 +239,17 @@ namespace Banshee.Sources
             set { ExpandedSchema.Set (value); }
         }
 
+        public PathPattern PathPattern { get; private set; }
+
+        protected void SetFileNamePattern (PathPattern pattern)
+        {
+            PathPattern = pattern;
+
+            var file_system = PreferencesPage.Add (new Section ("file-system", Catalog.GetString ("File Organization"), 5));
+            file_system.Add (new SchemaPreference<string> (pattern.FolderSchema, Catalog.GetString ("Folder hie_rarchy")));
+            file_system.Add (new SchemaPreference<string> (pattern.FileSchema, Catalog.GetString ("File _name")));
+        }
+
         public virtual void Dispose ()
         {
             if (Application.ShuttingDown)
@@ -241,7 +276,7 @@ namespace Banshee.Sources
             DatabaseTrackModel.AddCondition (String.Format ("CoreTracks.PrimarySourceID = {0}", DbId));
 
             primary_sources[DbId] = this;
-            
+
             // Load our playlists and smart playlists
             foreach (PlaylistSource pl in PlaylistSource.LoadAll (this)) {
                 AddChildSource (pl);
@@ -312,7 +347,7 @@ namespace Banshee.Sources
             OnTracksChanged ();
         }
 
-        internal void NotifyTracksDeleted ()
+        public void NotifyTracksDeleted ()
         {
             OnTracksDeleted ();
         }
@@ -352,15 +387,6 @@ namespace Banshee.Sources
             get { return true; }
         }
 
-        public override void SetParentSource (Source source)
-        {
-            if (source is PrimarySource) {
-                throw new ArgumentException ("PrimarySource cannot have another PrimarySource as its parent");
-            }
-
-            base.SetParentSource (source);
-        }
-
         protected override void OnTracksAdded ()
         {
             ThreadAssist.SpawnFromMain (delegate {
@@ -408,7 +434,7 @@ namespace Banshee.Sources
         {
             OnTracksDeleted ();
         }
-        
+
         protected virtual void PurgeTracks ()
         {
             ServiceManager.DbConnection.Execute (purge_tracks_command, DbId);
@@ -438,7 +464,10 @@ namespace Banshee.Sources
                 Log.WarningFormat ("Cannot delete all tracks from {0} via primary source {1}", source, this);
                 return;
             }
-            
+
+            if (source.Count < 1)
+                return;
+
             ThreadAssist.SpawnFromMain (delegate {
                 CachedList<DatabaseTrackInfo> list = CachedList<DatabaseTrackInfo>.CreateFromModel (source.DatabaseTrackModel);
                 DeleteTrackList (list);
@@ -447,10 +476,10 @@ namespace Banshee.Sources
 
         protected override void DeleteSelectedTracks (DatabaseTrackListModel model)
         {
-            if (model == null) {
+            if (model == null || model.Count < 1) {
                 return;
             }
-            
+
             ThreadAssist.SpawnFromMain (delegate {
                 CachedList<DatabaseTrackInfo> list = CachedList<DatabaseTrackInfo>.CreateFromModelSelection (model);
                 DeleteTrackList (list);
@@ -461,6 +490,7 @@ namespace Banshee.Sources
         {
             is_deleting = true;
             DeleteTrackJob.Total += (int) list.Count;
+            List<DatabaseTrackInfo> skip_deletion = null;
 
             // Remove from file system
             foreach (DatabaseTrackInfo track in list) {
@@ -471,7 +501,12 @@ namespace Banshee.Sources
 
                 try {
                     DeleteTrackJob.Status = String.Format ("{0} - {1}", track.ArtistName, track.TrackTitle);
-                    DeleteTrack (track);
+                    if (!DeleteTrack (track)) {
+                        if (skip_deletion == null) {
+                            skip_deletion = new List<DatabaseTrackInfo> ();
+                        }
+                        skip_deletion.Add (track);
+                    }
                 } catch (Exception e) {
                     Log.Exception (e);
                     ErrorSource.AddMessage (e.Message, track.Uri.ToString ());
@@ -490,8 +525,16 @@ namespace Banshee.Sources
                 delete_track_job = null;
             }
 
+            if (skip_deletion != null) {
+                list.Remove (skip_deletion);
+                skip_deletion.Clear ();
+                skip_deletion = null;
+            }
+
             // Remove from database
-            ServiceManager.DbConnection.Execute (remove_list_command, DateTime.Now, list.CacheId, list.CacheId);
+            if (list.Count > 0) {
+                ServiceManager.DbConnection.Execute (remove_list_command, DateTime.Now, list.CacheId, list.CacheId);
+            }
 
             ThreadAssist.ProxyToMain (delegate {
                 OnTracksDeleted ();
@@ -500,9 +543,18 @@ namespace Banshee.Sources
             });
         }
 
-        protected virtual void DeleteTrack (DatabaseTrackInfo track)
+        protected virtual bool DeleteTrack (DatabaseTrackInfo track)
         {
-            throw new Exception ("PrimarySource DeleteTrack method not implemented");
+            if (!track.Uri.IsLocalPath)
+                throw new Exception ("Cannot delete a non-local resource: " + track.Uri.Scheme);
+
+            try {
+                Banshee.IO.Utilities.DeleteFileTrimmingParentDirectories (track.Uri);
+            } catch (System.IO.FileNotFoundException) {
+            } catch (System.IO.DirectoryNotFoundException) {
+            }
+
+            return true;
         }
 
         public override bool AcceptsInputFromSource (Source source)
@@ -528,12 +580,13 @@ namespace Banshee.Sources
             }
             return true;
         }
-        
+
         public override bool AddAllTracks (Source source)
         {
-            if (!AcceptsInputFromSource (source) || source.Count == 0)
+            if (!AcceptsInputFromSource (source) || source.Count == 0) {
                 return false;
-            
+            }
+
             DatabaseTrackListModel model = (source as ITrackModelSource).TrackModel as DatabaseTrackListModel;
             CachedList<DatabaseTrackInfo> cached_list = CachedList<DatabaseTrackInfo>.CreateFromModel (model);
             if (ThreadAssist.InMainThread) {
@@ -544,6 +597,12 @@ namespace Banshee.Sources
             return true;
         }
 
+        private static HyenaSqliteCommand get_track_id_cmd = new HyenaSqliteCommand ("SELECT TrackID FROM CoreTracks WHERE PrimarySourceId = ? AND Uri = ? LIMIT 1");
+        public int GetTrackIdForUri (string uri)
+        {
+            return ServiceManager.DbConnection.Query<int> (get_track_id_cmd, DbId, new SafeUri (uri).AbsoluteUri);
+        }
+
         private bool is_adding;
         public bool IsAdding {
             get { return is_adding; }
@@ -573,7 +632,7 @@ namespace Banshee.Sources
                     IncrementAddedTracks ();
                     break;
                 }
-                
+
                 if (track == null) {
                     IncrementAddedTracks ();
                     continue;
@@ -602,19 +661,20 @@ namespace Banshee.Sources
                     finished = true;
                     add_track_job = null;
                 } else {
-                    add_track_job.Status = String.Empty;
                     if (add_track_job.Completed % 10 == 0)
                         notify = true;
                 }
             }
 
+            if (finished) {
+                is_adding = false;
+            }
+
             if (notify || finished) {
-                Banshee.Base.ThreadAssist.ProxyToMain (delegate {
-                    OnTracksAdded ();
-    
-                if (finished)
-                    OnUserNotifyUpdated ();
-                });
+                OnTracksAdded ();
+                if (finished) {
+                    ThreadAssist.ProxyToMain (OnUserNotifyUpdated);
+                }
             }
         }
 
@@ -624,10 +684,10 @@ namespace Banshee.Sources
             set { delay_add_job = value; }
         }
 
-        private bool delay_delete_jbo = true;
+        private bool delay_delete_job = true;
         protected bool DelayDeleteJob {
-            get { return delay_delete_jbo; }
-            set { delay_delete_jbo = value; }
+            get { return delay_delete_job; }
+            set { delay_delete_job = value; }
         }
 
         private BatchUserJob add_track_job;
@@ -636,8 +696,10 @@ namespace Banshee.Sources
                 lock (this) {
                     if (add_track_job == null) {
                         add_track_job = new BatchUserJob (String.Format (Catalog.GetString (
-                            "Adding {0} of {1} to {2}"), "{0}", "{1}", Name), 
+                            "Adding {0} of {1} to {2}"), "{0}", "{1}", Name),
                             Properties.GetStringList ("Icon.Name"));
+                        add_track_job.SetResources (Resource.Cpu, Resource.Database, Resource.Disk);
+                        add_track_job.PriorityHints = PriorityHints.SpeedSensitive | PriorityHints.DataLossIfStopped;
                         add_track_job.DelayShow = DelayAddJob;
                         add_track_job.CanCancel = true;
                         add_track_job.Register ();
@@ -655,6 +717,8 @@ namespace Banshee.Sources
                         delete_track_job = new BatchUserJob (String.Format (Catalog.GetString (
                             "Deleting {0} of {1} From {2}"), "{0}", "{1}", Name),
                             Properties.GetStringList ("Icon.Name"));
+                        delete_track_job.SetResources (Resource.Cpu, Resource.Database);
+                        delete_track_job.PriorityHints = PriorityHints.SpeedSensitive | PriorityHints.DataLossIfStopped;
                         delete_track_job.DelayShow = DelayDeleteJob;
                         delete_track_job.Register ();
                     }
